#ifndef HPP_HIK_SLACK_SOLVER_H
#define HPP_HIK_SLACK_SOLVER_H

#include "MatrixAbstractLayer/MatrixAbstractLayer.h"
#include "gikTask/jrlLinearConstraint.h"

/**
Mixture of Equality and Inequality Linear tasks solver.
Class design and implementation inspired from ChppQPsolved belonging to package hppQPsolver developed by Dr.Fumio KANEHIRO (AIST) within of the Associated International Laboratory: Joint Japanese-French Robotics Laboratory (JRL)
 */
class ChppHikSlackSolver
{
public:
    /**
    \brief Constructor
    \param i_ParameterNumber the number of parameters (excluding slack variables)
    \note number of parameters of entered hard constraints must be equal to  \param i_HardConstraintParameterNumber
     */
    ChppHikSlackSolver(unsigned int i_ParameterNumber);

    /**
        \brief Destructor
     */
    ~ChppHikSlackSolver();

    /**
        \name Tasks
        @{
     */

    /**
    \brief Get the number of linear inequaliy tasks
    */
    unsigned int inequalityTaskCount() const;
    /**
    \brief Get the number of linear inequaliy tasks
     */
    unsigned int equalityTaskCount() const;

    /**
    \brief Add equality or inequality tasks to be solved
    \param i_linearSystem an object of class CjrlLinearConstraint containg the linear system
    \param i_isEquality type of the linear system: equality or inequality
    \return false for mismatching number of parameters
     */
    bool addTasks(CjrlLinearConstraint *i_linearSystem, bool i_isEquality);


    /**
    \brief clear the slackened constraints
    \note objects of CjrlGikStateConstraint are not deleted. 
     */
    void clearTasks();

    /**
        @}
     */


    /**
    \name Constraints
     @{
     */
    /**
    \brief the number of linear equality constraints
    \return the number of linear equality constraints
    \note The number of constraints is not the length of array which contains CjrlGikStateConstraint objects. It's sum of dimensions of CjrlGikStateConstraint objects.
     */
    unsigned int equalityConstraintCount() const;

    /**
    \brief the number of linear inequality constraints
    \return the number of linear inequality constraints
    \note The number of constraints is not the length of array which contains CjrlGikStateConstraint objects. It's sum of dimensions of CjrlGikStateConstraint objects.
     */
    unsigned int inequalityConstraintCount() const;

    /**
    \brief Add linear constraints for problem
    \param i_linearSystem linear system
    \param i_isEquality type of the linear system: equality or inequality
    \return false for mismatching number of parameters
     */
    bool addConstraints(CjrlLinearConstraint *i_linearSystem, bool i_isEquality);

    /**
    \brief clear hard constraints
    \note objects of CjrlGikStateConstraint are not deleted. 
     */
    void clearConstraints();

    /**
    \name Solve
     @{
     */

    /**
    \brief Solve problem 
    \param io_solution initial guess is passed and overwritten by the computed solution 
    \param outSlackVariables of size slackCount() is used to write the resulting slack varialbe values. Order is: equality constraint then inequality constraint
    \return 0 if successful, -1000 if io_solution of wrong size, refer CFSQP manual otherwise
     */
    int solve(vectorN& io_solution, vectorN& outSlackVariables);

    /**
    @}
     */

    /**
    \name bound
    @{
     */

    /**
    \brief clear bounds of parameters with default bounds
     */
    void clearBounds();

    /**
    \brief set a default upper bound of \e i_rank th parameter. This value is used by clearBounds(). Initial value is 1e10.
    \param i_rank rank of parameter
    \param i_bound default upper bound
     */
    void defaultUpperBound(unsigned int i_rank, double i_bound);

    /**
    \brief set a default lower bound of \e i_rank th parameter. This value is used by clearBounds(). Initial value is -1e10
    \param i_rank rank of parameter
    \param i_bound default lower bound
     */
    void defaultLowerBound(unsigned int i_rank, double i_bound);

    /**
    \brief set an upper bound of \e i_rank th parameter. If the current upper bound is smaller than \e i_bound, the upper bound is not updated.
    \param i_rank rank of parameter
    \param i_bound upper bound
     */
    void upperBound(unsigned int i_rank, double i_bound);

    /**
    \brief set a lower bound of \e i_rank th parameter. If the current lower bound is bigger than \e i_bound, the lower bound is not updated. 
    \param i_rank rank of parameter
    \param i_bound lower bound
     */
    void lowerBound(unsigned int i_rank, double i_bound);

    /**
    \brief get an upper bound of \e i_rank th parameter
    \return upper bound
     */
    double upperBound(unsigned int i_rank) const;

    /**
    \brief get a lower bound of \e i_rank th parameter
    \return lower bound
     */
    double lowerBound(unsigned int i_rank) const;

    /**
    @}
     */

     /**
     \name For internal use
    @{
      */
    const std::vector<CjrlLinearConstraint*>& equalityTasks();
    bool valueOfConstraint(unsigned int i_rank, double *i_x, double& outValue);
    bool gradientOfConstraint(unsigned int i_rank, double* outVector);
     /**
    @}
      */

private:
    std::vector< std::pair<CjrlLinearConstraint*,unsigned int> > attEqConstraints, attIneqConstraints;
    std::vector< std::pair<CjrlLinearConstraint*,unsigned int> > attEqTasks, attIneqTasks;
    std::vector<CjrlLinearConstraint*> attEqTaskBlocs;

    unsigned int attVariableCount, attParamCount;
    unsigned int attEqTaskSize, attIneqTaskSize, attEqConstSize, attIneqConstSize;

    vectorN attFullState, attParams, tempdq,tempro;

    int nf, neqn,nineqn, mode,iprint,miter;
    int ncsrl,ncsrn,nfsr,mesh_pts[1];
    double bigbnd,eps,epsneq,udelta;
    double *f,*g,*lambda,*bl,*bu;
    vectorN attDefaultUBound,attDefaultLBound,attUBound,attLBound;
};
#endif
