#ifndef HPP_SOLVER_H
#define HPP_SOLVER_H

#include "hpphik/hppHikSlackSolver.hh"
#include "hpphik/hppHikLinearSystem.hh"

/**
\brief This object is the equality and inequality task solver. First an initial feasible set can be defined through methods addEqPreConstraint and addIneqPreConstraint, then initFeasibleSet must be called.
In decreasing priority, the tasks are solved through calls to solveIneq or solveEq depending on their types. After each solve operation, the feasible set has to be updated via updateFeasibleSet (except for the last one, not necesseary if one is only interested in the solution).
The feasible set can be retrieved via currentFeasibleSetEquality and currentFeasibleSetInequality.
*/
class ChppHikSolverBasic
{
public:
    /**
    \brief Constructor
    \param inNparam number of optimization parameters
    */
    ChppHikSolverBasic(unsigned int inNparam);
    
    /**
    \brief Get the number of parameters of optimization
    */
    unsigned int numberParams() {return attNParam;};

    /**
    \brief Destructor
     */
    virtual ~ChppHikSolverBasic();

    /**
    \name Bounds
    @{
    */
    /**
    \brief Upper bound on a control variable equivalent to a first priority inequality task
     */
    void upperBound(unsigned int i_rank, double i_bound);
    /**
    \brief Lower bound on a control variable equivalent to a first priority inequality task
     */
    void lowerBound(unsigned int i_rank, double i_bound);
    /**
    \brief Default lower bound on a control variable equivalent to a first priority inequality task
     */
    void defaultLowerBound(unsigned int i_rank, double i_bound);
    /**
    \brief Default upper bound on a control variable equivalent to a first priority inequality task
     */
    void defaultUpperBound(unsigned int i_rank, double i_bound);
    /**
    \brief Restore QP solver's default bounds
     */
    void clearBounds();
    /**
    @}
     */

    /**
    \name Initial Constraints
    @{
     */
    /**
    \brief Add equality pre-contraints (to define initial feasible set)
    */
    void addEqPreConstraint(CjrlLinearConstraint& inConstraint);
    /**
    \brief Add inequality pre-contraints (to define initial feasible set)
     */
    void addIneqPreConstraint( CjrlLinearConstraint& inConstraint);
    /**
    \brief Remove constraints added through addIneqConstraint and addEqConstraint.
    Effective on optimization problem only after a call to initFeasibleSet()
     */
    void removePreConstraints();
    /**
    @}
    */
    /**
    \brief Set active parameters by setting a mask
    \param inMask binary vector of size inNparam : 0 for disabled parameter and 1 for active
     */
    virtual bool setMask(const vectorN& inMask);

    /**
    \brief Reset feasible set to one defined by entered pre-constraints
     */
    void initFeasibleSet();
    
    
    /**
    \name Solving by priority stage
    @{
    */
    
    /**
    \brief Clear the vector of tasks to be solved.
    */
    void clearTasks();
    
    /**
    \brief Add a task to be solved within current feasible set. The added tasks share the same priority and can be of equality and/or inequality type.
    \return false if task jacobian does not match this solver's number of parameters
     */
    bool addTask(CjrlLinearConstraint& inTask, bool inIsEqualityTask);
    
    /**
    \brief Solve tasks
     */
    int solve(const vectorN& inInitialGuess, double inDampingFactor = 0);
    
    /**
    \brief Get tasks solution
     */
    const vectorN& solution();
    
    /**
    \brief Update feasible set based on current stage's set of tasks and solution.

     */
    void updateFeasibleSet();

    /**
    \brief Get current feasible set's equality part
     */
    CjrlLinearConstraint& currentFeasibleSetEquality();

    /**
    \brief Get current feasible set's inequality part
     */
    CjrlLinearConstraint& currentFeasibleSetInequality();
    /**
    @}
     */


protected:
    void clearConstraintsInSolver();
    void loadConstraintsInSolver();

    unsigned int attNParam;
    ChppHikSlackSolver* attSlackSolver;
    std::vector<CjrlLinearConstraint*> attEqConstraints,attIneqConstraints;
    ChppHikLinearSystem *attEqConstraint, *attIneqConstraint, *attIneqObjective, *attEqObjective;
    vectorN tempb, attMask, attSolution,attSlacks;
    double attThresh,attDampingFactor;
    std::vector< std::pair<CjrlLinearConstraint*,bool> > attTasks;
};

#endif
